#include "LED_SEG\LED_SEG.h"
#include "UART\uart.h"
#include "QN8027_DRV\QN8027_DRV.h"
#include "AT24C02_DRV\AT24C02_DRV.h"
#include "common\common.h"

sbit BtnFreqInc = P3^3;
sbit BtnFreqDec = P3^2;

#define FREQ_STORED_ADDR1 0xFE
#define FREQ_STORED_ADDR2 0xFF


void Change_TX_Freq(unsigned short TargetFreq)					//The parameter "TargetFreq" is the actual Frequency multiplied by 10.
{
	unsigned char StrTargetFreq[8] = "000.0\r\n\0";

	UartSendString("Frequency: ");
	StrTargetFreq[0] = TargetFreq/1000 + '0';
	StrTargetFreq[1] = (TargetFreq%1000)/100 + '0';
	StrTargetFreq[2] = (TargetFreq%100)/10 + '0';
	StrTargetFreq[4] = TargetFreq%10 + '0';
	UartSendString(StrTargetFreq);

	QN8027_SetFrequency(TargetFreq);
	SegDisplayFreq(TargetFreq);														//Display the frequency on the segment displays.

	AT24C02_ByteWR(FREQ_STORED_ADDR1, TargetFreq >> 8);		//Store current transmit frequency.
	Delay_us(600);							    											//AT24C02 has a maximum write cycle time of 5 ms.
	AT24C02_ByteWR(FREQ_STORED_ADDR2, TargetFreq&0x00FF);
}

void UART_Set_TX_Freq(void)															//For example, transmit frequency will be set to 89.6MHz if string "f0896" is received. 
{
	static unsigned short TargetFreq = 0;
	unsigned char UartBuf;
	static unsigned char UartRecvStatus = 0;

	if(IsUartReceived())
	{
		UartReadChar(&UartBuf);
		switch(UartRecvStatus)
		{
			case 0:
			{
				if(UartBuf == 'f')
				{
					TargetFreq = 0;
					UartSendChar('f');
					UartRecvStatus = 1;
				}
				break;
			}
			case 1:
			{
				if((UartBuf >= '0')&&(UartBuf <= '9'))
				{
					TargetFreq += (UartBuf-'0')*1000;
					UartSendChar(UartBuf);
					UartRecvStatus = 2;
				}
				break;
			}
			case 2:
			{
				if((UartBuf >= '0')&&(UartBuf <= '9'))
				{
					TargetFreq += (UartBuf-'0')*100;
					UartSendChar(UartBuf);
					UartRecvStatus = 3;
				}
				break;
			}
			case 3:
			{
				if((UartBuf >= '0')&&(UartBuf <= '9'))
				{
					TargetFreq += (UartBuf-'0')*10;
					UartSendChar(UartBuf);
					UartRecvStatus = 4;
				}
				break;
			}
			case 4:
			{
				if((UartBuf >= '0')&&(UartBuf <= '9'))
				{
					TargetFreq += (UartBuf-'0');
					UartSendChar(UartBuf);
					UartSendString("\r\n");
					if((TargetFreq >= 760)&&(TargetFreq <= 1080))
						Change_TX_Freq(TargetFreq);
					else
						UartSendString("Illegal input.\r\n");
					UartRecvStatus = 0;
				}
				break;
			}
		}
	}	 
}

int main(void)
{
	unsigned char QN8027_ID;
	unsigned char QN8027_Rev_Num[5] = "00\r\n\0";
	unsigned short StoredFreq;
	unsigned short TargetFreq;

	EA = 1;												//Global Interrupt Enable
	
	UartInit();
	UartSendString("*QN8027 FM Transmitter*\r\n");

	SegDisplayInit();

	QN8027_ID = QN8027_CheckID();
	if(QN8027_ID)
	{
		UartSendString("QN8027 detected\r\n");
		UartSendString("Chip major revision number: ");
		QN8027_Rev_Num[0] = QN8027_ID/160 + '0';
		QN8027_Rev_Num[1] = (QN8027_ID >> 4) % 10 + '0';
		UartSendString(QN8027_Rev_Num);
		UartSendString("Chip minor revision number: ");
		QN8027_Rev_Num[0] = '0';
		QN8027_Rev_Num[1] = (QN8027_ID & 0x03) + '0';
		UartSendString(QN8027_Rev_Num);

		QN8027_Init();

		StoredFreq = AT24C02_ByteRD(FREQ_STORED_ADDR1);
		StoredFreq = (StoredFreq << 8)| AT24C02_ByteRD(FREQ_STORED_ADDR2);
		if((StoredFreq >= 760)&&(StoredFreq <= 1080))
		{
			QN8027_SetFrequency(StoredFreq);
			SegDisplayFreq(StoredFreq);
		}
		else
		{
			QN8027_SetFrequency(1068);
			SegDisplayFreq(1068);
		}
		
		while(1)
		{
			if(BtnFreqInc == 0)				//If the frequency increase button is pressed
			{
				Delay_us(5000);					//Debouncing
				TargetFreq = QN8027_GetCurrentFrequency();
				while(BtnFreqInc == 0)
				{
					if(TargetFreq < 1080)
					{
						TargetFreq++;
						SegDisplayFreq(TargetFreq);
						Delay_us(25000);
					}
					if(BtnFreqInc == 1)		//If the frequency increase button is released
					{
						Delay_us(5000);			//Debouncing
						if(BtnFreqInc == 1)
						{
							Change_TX_Freq(TargetFreq);
							break;
						}
					}
				}
			}
			else if(BtnFreqDec == 0)	//If the frequency decrease button is pressed
			{
				Delay_us(5000);
				TargetFreq = QN8027_GetCurrentFrequency();
				while(BtnFreqDec == 0)
				{
					if(TargetFreq > 760)
					{
						TargetFreq--;
						SegDisplayFreq(TargetFreq);
						Delay_us(25000);
					}
					if(BtnFreqDec == 1)		//If the frequency decrease button is released
					{
						Delay_us(5000);			//Debouncing
						if(BtnFreqDec == 1)
						{
							Change_TX_Freq(TargetFreq);
							break;
						}
					}
				}
			}
			
			UART_Set_TX_Freq();
		}
	}
	else
	{
		UartSendString("QN8027 not found.\r\n");
		SegDisplayFreq(0);
	}
	
	return 0;
}